%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% File name: ExtendedKalman2D.m
%
% Description: The script executes the recursive Extended Kalman algorithm of the
%              reference publication, for estimation of the periodic system
%              behind the measured signal. Note that the algorithm assumes
%              a one dimensional measurement, see Generate2DData.m. All
%              estimated parameters are for the second underlying state equation.
%
% Reference publications: T. Wigren, E. Abd-Elrady and T. Sderstrm, 
%                         "Harmonic signal analysis with Kalman filters 
%                         using periodic orbits of nonlinear ODEs", 
%                         Proc. ICASSP 2003, Hong Kong, China, vol. VI, pp.
%                         669-672, 2003.
%
%                         http://www.it.uu.se/katalog/tw/research/periodicSignalEstimation
%
% Change log: 2003-03-04 - First verision ready. 
%             2013-09-01 - Restructured, and documentation added
%             2013-09-29 - Minor corrections and comments added
%
% Copyright: Torbjrn Wigren, 2013-09-29.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% Model structure definition. Dimensions of polynomial right hand sides 
% - in the first and second independent variable.
%

DimensionF2X1=2;
DimensionF2X2=1;

%
% Sizes of vectors and matrises
%

NumberMeasurements=size(z,1);
Theta2Size=(DimensionF2X1+1)*(DimensionF2X2+1);
ParameterVectorSize=Theta2Size+2;

%
% Initial values
%

x1o=-0.5;
x2o=0.5;
if exist('theta')
    theta2o=theta;
else
    theta2o=zeros(Theta2Size,1);
end
X=[x1o;x2o;theta2o];
thetaSave=[];
eSave=[];
P=10*eye(ParameterVectorSize(1,1));% Initial value for P

%
% Estimator parameter settings
%

H=[1 zeros(1,ParameterVectorSize(1,1)-1)];
R2=1;
R1=0.00001*eye(ParameterVectorSize(1,1));

%
% Main extended Kalman filter loop
%

for t=2:NumberMeasurements-2   
    S=H*P*H'+R2; % Standard EKF equations
    K=P*H'*inv(S);
    X=X+K*(z(t,1)-H*X);
    thetaSave=[thetaSave X(3:ParameterVectorSize)]; % Save running parameter estimate  
    eSave=[eSave; z(t,1)-H*X]; % Save running prediction error
    P=P-P*H'*inv(S)*H*P;
    deltaPhi2=[]; % To propagate the state and the covariance, the linearized system matrix is needed, as well as the normal state dependen one
    DifPhi2X1=[];
    DifPhi2X2=[];
    for i=0:DimensionF2X1
      for j=0:DimensionF2X2
         deltaPhi2=[deltaPhi2 X(1)^i*X(2)^j];
	     DifPhi2X1=[DifPhi2X1 i*X(1)^(i-1)*X(2)^j];
	     DifPhi2X2=[DifPhi2X2 j*X(1)^i*X(2)^(j-1)];
      end;
    end;
   deltaPhi2=(SamplingInstances(1,t+1)-SamplingInstances(1,t))*deltaPhi2;
   DifPhi2X1=(SamplingInstances(1,t+1)-SamplingInstances(1,t))*DifPhi2X1;
   DifPhi2X2=(SamplingInstances(1,t+1)-SamplingInstances(1,t))*DifPhi2X2;
   F=[1 Tsampling zeros(1,Theta2Size(1,1));0 1 deltaPhi2;
      zeros(Theta2Size(1,1),1) zeros(Theta2Size(1,1),1) eye(Theta2Size(1,1))];   
   tildF=[1 Tsampling zeros(1,Theta2Size(1,1));
          DifPhi2X1*X(3:ParameterVectorSize(1,1)) 1+DifPhi2X2*X(3:ParameterVectorSize(1,1)) deltaPhi2;
          zeros(Theta2Size(1,1),1) zeros(Theta2Size(1,1),1) eye(Theta2Size(1,1))];
   X=F*X; % State propagation
   P=tildF*P*tildF'+R1; % Covariance propagation
end;

%
% End of main algorithmic loop
%

theta=X(3:ParameterVectorSize(1,1)) % EKF parameter estimate

