%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% File name: Kalman2D.m
%
% Description: The script executes the recursive Kalman algorithm of the
%              reference publication, for estimation of the periodic system
%              behind the measured signal. Note that the algorithm assumes
%              a one dimensional measurement, see Generate2DData.m. All
%              estimated parameters are for the second underlying state equation.
%
% Reference publications: T. Wigren, E. Abd-Elrady and T. Sderstrm, 
%                         "Harmonic signal analysis with Kalman filters 
%                         using periodic orbits of nonlinear ODEs", 
%                         Proc. ICASSP 2003, Hong Kong, China, vol. VI, pp.
%                         669-672, 2003.
%
%                         http://www.it.uu.se/katalog/tw/research/periodicSignalEstimation
%
% Change log: 2003-02-27 - First verision ready. 
%             2013-09-01 - Restructured, and documentation added
%             2013-09-29 - Minor corrections and comments added
%
% Copyright: Torbjrn Wigren, 2013-09-29.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% Model structure definition. Dimensions of polynomial right hand sides 
% - in the first and second independent variable.
%

DimensionF2X1=2;
DimensionF2X2=1;

%
% Sizes of vectors and matrises
%

NumberMeasurements=size(z,1);
Theta2Size=(DimensionF2X1+1)*(DimensionF2X2+1);
ParameterVectorSize=Theta2Size+2;

%
% Initial values
%

x1o=-0.5;
x2o=0.5;
theta2o=zeros(Theta2Size,1);
X=[x1o;x2o;theta2o];
thetaSave=[];
eSave=[];
P=1000*eye(ParameterVectorSize(1,1));% Initial value for P

%
% Estimator parameter settings
%

H=[1 zeros(1,ParameterVectorSize(1,1)-1)];
R2=1;
R1=0.0001*eye(ParameterVectorSize(1,1));

%
% Input signal differentiation, see reference publication for
% details
%

zdot=0*z;
for t=2:NumberMeasurements-1
   zdot(t,1)=(z(t+1,1)-z(t-1,1))/(SamplingInstances(1,t+1)-SamplingInstances(1,t-1)); % Euler differentiation
end;
zdot(1,1)=zdot(2,1); % Boundary correction
zdot(NumberMeasurements,1)=zdot(NumberMeasurements-1,1); % Boundary correction
z=[z zdot];

%
% Main Kalman filter algorithm loop
%

for t=2:NumberMeasurements-2
   S=H*P*H'+R2; % Standard kalman filter equations...
   K=P*H'*inv(S);
   X=X+K*(z(t,1)-H*X);
   thetaSave=[thetaSave X(3:ParameterVectorSize)]; % Save running parameter estimate
   eSave=[eSave;z(t,1)-H*X]; % Save running prediction error
   P=P-P*H'*inv(S)*H*P;
   deltaPhi2=[]; % The state and covariance propagation requires building up the linearized system matrix...
   for i=0:DimensionF2X1
      for j=0:DimensionF2X2
         deltaPhi2=[deltaPhi2 z(t,1)^i*z(t,2)^j];
      end;
    end;
   deltaPhi2=(SamplingInstances(1,t+1)-SamplingInstances(1,t))*deltaPhi2;
   F=[1 Tsampling zeros(1,Theta2Size(1,1));
      0 1 deltaPhi2;
      zeros(Theta2Size(1,1),1) zeros(Theta2Size(1,1),1) eye(Theta2Size(1,1))]; % The linearized system matrix
   X=F*X; % State propagation
   P=F*P*F'+R1; % Covariance propagation
end;

%
% End of main algorithmic loop
%

z=z(:,1); % Remove differentiated signal, to prepare for further processing
theta=X(3:ParameterVectorSize(1,1)) % The estimated parameters, also used for initialization of 2DExtendedKalman

