%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% File name: LeastSquaresDifferentiated2D.m
%
% Description: The script executes the least squares algorithm of the
%              reference publication, for estimation of the periodic system
%              behind the measured signal. Note that the algorithm assumes
%              a one dimensional measurement, the other is generated by numerical 
%              differentiation, see Generate2DData.m. The estimated parameters 
%              are for the first as well as for the second underlying state equation.
%
% Reference publications: T. Wigren, E. Abd-Elrady and T. Sderstrm, 
%                         "Least squares harmonic signal analysis using 
%                         periodic orbits of ODEs", Prep. 13:th IFAC Symposium 
%                         on System Identification , Rotterdam, The Netherlands, 
%                         pp.1584-1589, August 27-29, 2003.
%
%                         http://www.it.uu.se/katalog/tw/research/periodicSignalEstimation
%
% Change log: 2003-03-04 - First verision ready. 
%             2013-09-28 - Restructured, and documentation added
%
% Copyright: Torbjrn Wigren, 2013-09-28.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% Model structure definition. Orders of polynomial right hand sides 
% - in the first and second independent variable, for the first and 
% second state equation
%

DimensionF1X1=1;
DimensionF1X2=1;
DimensionF2X1=2;
DimensionF2X2=1;

%
% Sizes of vectors and matrices
%

NumberOfMeasurements=size(z,1);

%
% Differentiation of input signal and extension of measurement vector
%

zdot=0*z;
for t=2:NumberOfMeasurements-1
   zdot(t,1)=(z(t+1,1)-z(t-1,1))/(SamplingInstances(1,t+1)-SamplingInstances(1,t-1));
end;
zdot(1,1)=zdot(2,1);
zdot(NumberOfMeasurements,1)=zdot(NumberOfMeasurements-1,1);
z=[z zdot];

%
% Intilalization of regression vectors
%

Y=[];
Phi=[];

%
% Build up of regression vectors
%

for t=2:NumberOfMeasurements-2
   ThisTime=SamplingInstances(1,t);
   NextTime=SamplingInstances(1,t+1);
   Y=[Y;z(t+1,:)'-z(t,:)'];
   deltaPhi1=[];
   deltaPhi2=[];
   for i=0:DimensionF1X1
      for j=0:DimensionF1X2
         deltaPhi1=[deltaPhi1 z(t,1)^i*z(t,2)^j];
      end;
   end;
   for i=0:DimensionF2X1
      for j=0:DimensionF2X2
         deltaPhi2=[deltaPhi2 z(t,1)^i*z(t,2)^j];
      end;
   end;
   deltaPhi1=(NextTime-ThisTime)*deltaPhi1;
   deltaPhi2=(NextTime-ThisTime)*deltaPhi2;
   Phi=[Phi;deltaPhi1 0*deltaPhi2;0*deltaPhi1 deltaPhi2];
end;

%
% Computation of LS parameter estimate
%

Theta=inv(Phi'*Phi)*Phi'*Y

%
% Recomputation of original z
% 

z=z(:,1);

%
% Make sure plotting is correct
%

clear eSave thetaSave;